@@ -2,6 +2,7 @@ package javax.jmdns.test;
 
 import static junit.framework.Assert.assertEquals;
 import static junit.framework.Assert.assertTrue;
+import static junit.framework.Assert.*;
 import static org.easymock.EasyMock.capture;
 import static org.easymock.EasyMock.createMock;
 import static org.easymock.EasyMock.createNiceMock;
@@ -11,7 +12,9 @@ import static org.easymock.EasyMock.verify;
 import java.io.IOException;
 import java.net.DatagramPacket;
 import java.net.InetAddress;
+import java.net.Inet6Address;
 import java.net.MulticastSocket;
+import java.net.NetworkInterface;
 import java.net.UnknownHostException;
 import java.util.Enumeration;
 import java.util.HashMap;
@@ -119,6 +122,54 @@ public class JmDNSTest {
         }
     }
 
+    @Test
+    public void testRegisterServiceTwice() throws IOException {
+        System.out.println("Unit Test: testRegisterService()");
+        JmDNS registry = null;
+        try {
+            registry = JmDNS.create();
+            registry.registerService(service);
+            // This should cause an exception
+            registry.registerService(service);
+            fail("Registering the same service info should fail.");
+        } catch (IllegalStateException exception) {
+            // Expected exception.
+        } finally {
+            if (registry != null) registry.close();
+        }
+    }
+
+    @Test
+    public void testUnregisterAndReregisterService() throws IOException, InterruptedException {
+        System.out.println("Unit Test: testUnregisterAndReregisterService()");
+        JmDNS registry = null;
+        try {
+            registry = JmDNS.create();
+            registry.registerService(service);
+
+            ServiceInfo[] services = registry.list(service.getType());
+            assertEquals("We should see the service we just registered: ", 1, services.length);
+            assertEquals(service, services[0]);
+
+            // now unregister and make sure it's gone
+            registry.unregisterService(services[0]);
+
+            // According to the spec the record disappears from the cache 1s after it has been unregistered
+            // without sleeping for a while, the service would not be unregistered fully
+            Thread.sleep(1500);
+
+            services = registry.list(service.getType());
+            assertTrue("We should not see the service we just unregistered: ", services == null || services.length == 0);
+
+            registry.registerService(service);
+            Thread.sleep(5000);
+            services = registry.list(service.getType());
+            assertTrue("We should see the service we just reregistered: ", services != null && services.length > 0);
+        } finally {
+            if (registry != null) registry.close();
+        }
+    }
+
     @Test
     public void testQueryMyService() throws IOException {
         System.out.println("Unit Test: testQueryMyService()");
@@ -148,6 +199,29 @@ public class JmDNSTest {
         }
     }
 
+    @Test
+    public void testListMyServiceIPV6() throws IOException {
+        System.out.println("Unit Test: testListMyServiceIPV6()");
+        JmDNS registry = null;
+        try {
+            InetAddress address = InetAddress.getLocalHost();
+            NetworkInterface interfaze = NetworkInterface.getByInetAddress(address);
+            for (Enumeration<InetAddress> iaenum = interfaze.getInetAddresses(); iaenum.hasMoreElements();) {
+                InetAddress interfaceAddress = iaenum.nextElement();
+                if (interfaceAddress instanceof Inet6Address) {
+                    address = interfaceAddress;
+                }
+            }
+            registry = JmDNS.create(address);
+            registry.registerService(service);
+            ServiceInfo[] services = registry.list(service.getType());
+            assertEquals("We should see the service we just registered: ", 1, services.length);
+            assertEquals(service, services[0]);
+        } finally {
+            if (registry != null) registry.close();
+        }
+    }
+
     @Test
     public void testListenForMyService() throws IOException {
         System.out.println("Unit Test: testListenForMyService()");
@@ -464,4 +538,4 @@ public class JmDNSTest {
             if (registry != null) registry.close();
         }
     }
-}
\ No newline at end of file
+}
